/*
 * Process-related convenience functions and types.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

#include <fsclient/file.h>
#include <systypes/base.h>



EXTERN_C_BEGIN

/* File access abstraction compatible with notifiable_base_t. */

typedef struct
{
  /* File object reference. */

  l4_cap_idx_t ref;

  /* Notification structure. */

  notifiable_t notifiable;

} process_t;



/* Principal functions. */

process_t *process_new(void);
void process_close(process_t *process);
long process_error(process_t *process);
void process_free(process_t *process);
void process_init(process_t *process);
long process_spawn(int argc, const char *argv[], file_t *writer, process_t **process);
long process_start(process_t *process, int argc, const char *argv[], file_t *writer);
long process_wait(process_t *process, notify_flags_t *flags, notify_values_t *values);

/* Notification support. */

notifiable_t *process_notifiable(process_t *process);
notify_flags_t process_notifications(process_t *process);
notify_values_t process_notification_values(process_t *process);

long process_notify_wait_process(process_t *process, notifier_t *notifier);
long process_notify_wait_processes(process_t **process, notifier_t *notifier);

EXTERN_C_END

// vim: tabstop=2 expandtab shiftwidth=2
